@@ -82,7 +82,10 @@ module Hledger.Data.Amount (
   costOfMixedAmount,
   divideMixedAmount,
   averageMixedAmounts,
+  isNegativeAmount,
   isNegativeMixedAmount,
+  isZeroAmount,
+  isReallyZeroAmount,
   isZeroMixedAmount,
   isReallyZeroMixedAmount,
   isReallyZeroMixedAmountCost,
