@@ -10,6 +10,8 @@ balances, and postings in each account.
 module Hledger.Data.Ledger
 where
 import qualified Data.Map as M
+-- import Data.Text (Text)
+import qualified Data.Text as T
 import Safe (headDef)
 import Test.HUnit
 import Text.Printf
@@ -37,7 +39,7 @@ nullledger = Ledger {
 
 -- | Filter a journal's transactions with the given query, then derive
 -- a ledger containing the chart of accounts and balances. If the
--- query includes a depth limit, that will affect the this ledger's
+-- query includes a depth limit, that will affect the ledger's
 -- journal but not the ledger's account tree.
 ledgerFromJournal :: Query -> Journal -> Ledger
 ledgerFromJournal q j = nullledger{ljournal=j'', laccounts=as}
@@ -72,7 +74,7 @@ ledgerLeafAccounts = filter (null.asubs) . laccounts
 
 -- | Accounts in ledger whose name matches the pattern, in tree order.
 ledgerAccountsMatching :: [String] -> Ledger -> [Account]
-ledgerAccountsMatching pats = filter (matchpats pats . aname) . laccounts
+ledgerAccountsMatching pats = filter (matchpats pats . T.unpack . aname) . laccounts -- XXX pack
 
 -- | List a ledger's postings, in the order parsed.
 ledgerPostings :: Ledger -> [Posting]
@@ -84,8 +86,8 @@ ledgerDateSpan :: Ledger -> DateSpan
 ledgerDateSpan = postingsDateSpan . ledgerPostings
 
 -- | All commodities used in this ledger.
-ledgerCommodities :: Ledger -> [Commodity]
-ledgerCommodities = M.keys . jcommoditystyles . ljournal
+ledgerCommodities :: Ledger -> [CommoditySymbol]
+ledgerCommodities = M.keys . jinferredcommodities . ljournal
 
 
 tests_ledgerFromJournal = [
