@@ -23,6 +23,7 @@ module Hledger.Data.RawOptions (
 where
 
 import Data.Maybe
+import qualified Data.Text as T
 import Safe
 
 import Hledger.Utils
@@ -32,7 +33,7 @@ import Hledger.Utils
 type RawOpts = [(String,String)]
 
 setopt :: String -> String -> RawOpts -> RawOpts
-setopt name val = (++ [(name, quoteIfNeeded val)])
+setopt name val = (++ [(name, quoteIfNeeded $ val)])
 
 setboolopt :: String -> RawOpts -> RawOpts
 setboolopt name = (++ [(name,"")])
@@ -45,7 +46,7 @@ boolopt :: String -> RawOpts -> Bool
 boolopt = inRawOpts
 
 maybestringopt :: String -> RawOpts -> Maybe String
-maybestringopt name = maybe Nothing (Just . stripquotes) . lookup name . reverse
+maybestringopt name = maybe Nothing (Just . T.unpack . stripquotes . T.pack) . lookup name . reverse
 
 stringopt :: String -> RawOpts -> String
 stringopt name = fromMaybe "" . maybestringopt name
