@@ -64,16 +64,12 @@ import Hledger.Read.Common (amountp, statusp, genericSourcePos)
 
 
 reader :: Reader
-reader = Reader format detect parse
-
-format :: String
-format = "csv"
-
--- | Does the given file path and data look like it might be CSV ?
-detect :: FilePath -> Text -> Bool
-detect f t
-  | f /= "-"  = takeExtension f == '.':format  -- from a file: yes if the extension is .csv
-  | otherwise = T.length (T.filter (==',') t) >= 2 -- from stdin: yes if there are two or more commas
+reader = Reader
+  {rFormat     = "csv"
+  ,rExtensions = ["csv"]
+  ,rParser     = parse
+  ,rExperimental = False
+  }
 
 -- | Parse and post-process a "Journal" from CSV data, or give an error.
 -- XXX currently ignores the string and reads from the file path
@@ -103,14 +99,12 @@ readJournalFromCsv mrulesfile csvfile csvdata =
 
   -- parse rules
   let rulesfile = fromMaybe (rulesFileFor csvfile) mrulesfile
-  created <- ensureRulesFileExists rulesfile
-  if created
-   then hPrintf stderr "creating default conversion rules file %s, edit this file for better results\n" rulesfile
-   else hPrintf stderr "using conversion rules file %s\n" rulesfile
-  rules_ <- liftIO $ runExceptT $ parseRulesFile rulesfile
-  let rules = case rules_ of
-              Right (t::CsvRules) -> t
-              Left err -> throwerr err
+  rulesfileexists <- doesFileExist rulesfile
+  when rulesfileexists $ hPrintf stderr "using conversion rules file %s\n" rulesfile
+  rules <-
+    if rulesfileexists
+    then liftIO (runExceptT $ parseRulesFile rulesfile) >>= either throwerr return
+    else return defaultRules
   dbg2IO "rules" rules
 
   -- apply skip directive
@@ -147,6 +141,11 @@ readJournalFromCsv mrulesfile csvfile csvdata =
   -- so that same-day txns' original order is preserved
       txns' | length txns > 1 && tdate (head txns) > tdate (last txns) = reverse txns
             | otherwise = txns
+
+  when (not rulesfileexists) $ do
+    hPrintf stderr "created default conversion rules file %s, edit this for better results\n" rulesfile
+    writeFile rulesfile $ T.unpack $ defaultRulesText rulesfile
+
   return $ Right nulljournal{jtxns=sortBy (comparing tdate) txns'}
 
 parseCsv :: FilePath -> String -> IO (Either Parsec.ParseError CSV)
@@ -196,22 +195,9 @@ rulesFileFor = (++ ".rules")
 csvFileFor :: FilePath -> FilePath
 csvFileFor = reverse . drop 6 . reverse
 
--- | Ensure there is a conversion rules file at the given path, creating a
--- default one if needed and returning True in this case.
-ensureRulesFileExists :: FilePath -> IO Bool
-ensureRulesFileExists f = do
-  exists <- doesFileExist f
-  if exists
-   then return False
-   else do
-     -- note Hledger.Utils.UTF8.* do no line ending conversion on windows,
-     -- we currently require unix line endings on all platforms.
-     writeFile f $ newRulesFileContent f
-     return True
-
-newRulesFileContent :: FilePath -> String
-newRulesFileContent f = unlines
-  ["# hledger csv conversion rules for " ++ csvFileFor (takeFileName f)
+defaultRulesText :: FilePath -> Text
+defaultRulesText csvfile = T.pack $ unlines
+  ["# hledger csv conversion rules for " ++ csvFileFor (takeFileName csvfile)
   ,"# cf http://hledger.org/manual#csv-files"
   ,""
   ,"account1 assets:bank:checking"
@@ -233,6 +219,12 @@ newRulesFileContent f = unlines
   ," account2 assets:bank:savings\n"
   ]
 
+defaultRules :: CsvRules
+defaultRules =
+  either
+    (error' "Could not parse the default CSV rules, this should not happen")
+    id
+    $ parseCsvRules "" $ defaultRulesText ""
 
 --------------------------------------------------------------------------------
 -- Conversion rules parsing
