@@ -26,10 +26,12 @@ module Hledger.Reports.BalanceReport (
 )
 where
 
-import Data.List (sort)
+import Data.List
+import Data.Ord
 import Data.Maybe
 import Data.Time.Calendar
 import Test.HUnit
+import qualified Data.Text as T
 
 import Hledger.Data
 import Hledger.Read (mamountp')
@@ -148,8 +150,11 @@ balanceReportValue :: Journal -> Day -> BalanceReport -> BalanceReport
 balanceReportValue j d r = r'
   where
     (items,total) = r
-    r' = dbg8 "balanceReportValue"
-         ([(n, n', i, mixedAmountValue j d a) |(n,n',i,a) <- items], mixedAmountValue j d total)
+    r' =
+      dbg8 "known market prices" (jmarketprices j) `seq`
+      dbg8 "report end date" d `seq`
+      dbg8 "balanceReportValue"
+        ([(n, n', i, mixedAmountValue j d a) |(n,n',i,a) <- items], mixedAmountValue j d total)
 
 mixedAmountValue :: Journal -> Day -> MixedAmount -> MixedAmount
 mixedAmountValue j d (Mixed as) = Mixed $ map (amountValue j d) as
@@ -166,17 +171,23 @@ amountValue j d a =
                 }
     Nothing -> a
 
--- | Find the market value, if known, of one unit of this commodity on
--- the given date, in the commodity in which it has most recently been
--- market-priced (ie the commodity mentioned in the most recent
--- applicable market price directive before this date).
+-- | Find the market value, if known, of one unit of this commodity (A) on
+-- the given valuation date, in the commodity (B) mentioned in the latest
+-- applicable market price. The latest applicable market price is the market
+-- price directive for commodity A with the latest date that is on or before
+-- the valuation date; or if there are multiple such prices with the same date,
+-- the last parsed.
 commodityValue :: Journal -> Day -> CommoditySymbol -> Maybe Amount
-commodityValue j d c
-    | null applicableprices = Nothing
-    | otherwise             = Just $ mpamount $ last applicableprices
+commodityValue j valuationdate c
+    | null applicableprices = dbg Nothing
+    | otherwise             = dbg $ Just $ mpamount $ last applicableprices
   where
-    applicableprices = [p | p <- sort $ jmarketprices j, mpcommodity p == c, mpdate p <= d]
-
+    dbg = dbg8 ("using market price for "++T.unpack c)
+    applicableprices =
+      [p | p <- sortBy (comparing mpdate) $ jmarketprices j
+      , mpcommodity p == c
+      , mpdate p <= valuationdate
+      ]
 
 
 
@@ -387,7 +398,7 @@ tests_balanceReport =
  ]
 
 Right samplejournal2 =
-  journalBalanceTransactions
+  journalBalanceTransactions False
     nulljournal{
       jtxns = [
         txnTieKnot Transaction{
